{MODEL FOR THE ABSOLUTE SMALLEST SCREEN SAVERS POSSIBLE USING DELPHI

 Researched, designed, developed, coded, tested, and documented by:

                            CHARLES LEE LANKFORD
                            108 MARSHALL ST. SW
                            HARTSELLE, AL  35640-4106

                            lankford@net-master.net

 This software is the source code for producing the smallest (size) screen
 savers that can possibly be created with Delphi.  The package, as it is
 distributed, contains both the source code and a demo final product.  The
 final product can, of course, be used as a screen saver, but has really
 been included to allow you to compare it's size, etc., with screen savers
 that you develop from this model.  The source code can be used to recreate
 the demo screen saver, or develop your own "tiny" sized screen savers.  To
 develop new screen savers from this model, simply follow the directions
 below.  The source code also illustrates the usage of various Windows API
 calls, as well as providing insight into certain Windows oriented coding
 techniques.

 Much research and development time and effort was spent on this software's
 creation, testing, etc.  As such, if you find this package helpful to you
 in your programming endeavors, I would certainly appreciate it if you
 would please send me a few bucks (dollars, greenbacks, etc.) to help cover
 my costs in time, effort, utilities, etc.  I will NOT specify ANY dollar
 amount, simply send whatever you think it is worth to you.  After all,
 Windows programmers are all "brothers under the skin," so to speak.  Many
 thanks to you, in advance, for your support.

 Please send payment ($U.S., cash, check, or money order) to:

                         C. L. LANKFORD
                         108 MARSHALL ST. SW
                         HARTSELLE, AL  35640-4106

 To begin a new screen saver: copy these original .DPR, .RES, and .OPT files
 to a new file basename (like temp.dpr, temp.res, temp.opt), open the new
 .DPR file with Delphi, change the program name, change the screen saver
 name to whatever you want it to be known as by Control Panel/DeskTop, make
 changes to the code areas indicated (and types, variables, constants, etc.)
 to do whatever you need, and then save the file to disk.  The original
 source code indicates areas that you should/should NOT modify, and is
 intended to provide you with a general model (and demo) from which you
 can create other screen savers with minimal effort, other than the actual
 drawing/graphics for your screen updates, etc.  Remember when you are
 creating your graphics stuff, that this screen saver (and ones you create
 with it) MUST use Windows API calls for drawing/graphics stuff.  To do
 otherwise will require the FORMS unit from Delphi (and subsequently add
 about 150K or more to the size of your screen saver).

 To create your screen saver: modify a copy of this source code per your
 screen saver coding requirements and name changes, compile the program
 to an .EXE file, and then rename the .EXE file to a .SCR file.  Note -
 the last step is worth repeating: rename the .EXE file to a .SCR file.
 This renaming of the .EXE file to a .SCR file is the key to having Windows
 recognize the program as a screen saver.

 To install your screen saver: copy the .SCR file to the Windows directory
 (typically "c:\windows"), from Program Manager open Control Panel, run the
 DeskTop application, select your screen saver from the screen saver list,
 press "Test" or "Setup" if you wish (to check it out), and press "OK" to
 activate it or press "Cancel" to abort changes.}

program CLSCRSVR;             {<--- replace with your program's real name}

{$D SCRNSAVE:CL Screen Saver} {<--- replace name, after ":", with real name}
       {Screen saver name to use in Control Panel Desktop screen saver list}

uses                              {add/change to suit your needs...}
     WinTypes,WinProcs,Messages;  {other Windows/Delphi stuff needed...}

const                             {add/change to suit your needs...}
     maxpens=300;  {# pens for drawing, etc., specific to THIS screen saver}

type                              {add/change to suit your needs...}
     penarray = array [1..maxpens] of hpen; {pen handles array, THIS scrsvr}

var                             {add/change to suit your needs...}
     pens:penarray;             {array of pen handles for THIS screen saver}
     blackfill:hpen;            {black pen, to clear graphics, THIS scrnsavr}
     blackbrush:hbrush;         {handle to a black brush, THIS screen saver}
     xmax,ymax:integer;         {max window coordinates, used by THIS scrnsavr}
     start_time:longint;        {time screen saver graphics were started, msec}
{DO NOT CHANGE/DELETE THESE THREE LINES, UNLESS YOU WANT TO MODIFY ALL ROUTINES}
     scrsvrhwnd:hwnd;           {GLOBAL handle to screen saver window created}
     scrsvrhdc:hdc;             {GLOBAL handle to scrsvr window device context}
     c:string;                  {command string parameter, used by main block}


{Screen Saver configuration/setup procedure.  You need to change the code
 in this procedure to do whatever configuring/setup you want to allow the
 user to do with your screen saver execution parameters.  Note - parameters
 that you allow the user to change should be stored somewhere (INI file, etc.)
 for retrieval by initialize_scr when the screen saver is actually executed.}

{Remember that, if you change this procedure's name or parameters, you will
 also need to modify the other routines which call it.  It is suggested that
 you leave the name, and parameters, alone, and provide for additional data
 stuff needed via global variables declared in the main program.}

procedure configure_scr;
begin                   {for a demo scrnsavr, we don't have any options...}
     messagebox(0,'This screen saver has no options that you can set.',
                  'CL Screen Saver',mb_ok+mb_iconinformation);
end;


{Screen Saver initialization procedure.  Reads INI stuff, sets execution
 parameters defined/stored by configure_scr, etc.  You may want to change
 the code in this procedure to do whatever initialization tasks you need,
 including changing the timeout interval from the default value, etc.  In
 short, this procedure should incorporate anything you allow the user to
 modify (via configure_scr) into the screen saver execution parameters.
 Note - setting timeout<=0 will produce a blank screen, screen saver.}

{Remember that, if you change this procedure's name or parameters, you will
 also need to modify the other routines which call it.  It is suggested that
 you leave the name, and parameters, alone, and provide for additional data
 stuff needed via global variables declared in the main program.}

procedure initialize_scr(var timeout:integer);
const
     maxpenwidth=100;      {max width of pens to be created + 1}
     maxpenRGB=256;        {max value for RGB components + 1}
var
     i:integer;            {loop index}
     temprgb:longint;      {RGB value}
begin
     timeout:=500;        {how often, in msec, to fire the timer...}
     xmax:=getsystemmetrics(sm_cxscreen);  {xmax = max screen width}
     ymax:=getsystemmetrics(sm_cyscreen);  {ymax = max screen height}
     blackfill:=createpen(ps_solid,2*xmax,0); {black, for screen clearing}
     blackbrush:=getstockobject(black_brush); {black brush...}
     selectobject(scrsvrhdc,blackbrush);      {select the brush into DC}
     SetTextColor(scrsvrhdc,$00FFFFFF);       {use white text color...}
     setbkmode(scrsvrhdc,transparent);
     randomize;                               {seed random number generator}
     for i:=1 to maxpens do                   {do for each pen...}
         begin
           temprgb:=$00000000;
           while (temprgb=$00000000) or (temprgb=$00FFFFFF) do  {if black/white}
                 temprgb:=getnearestcolor(scrsvrhdc,rgb(random(maxpenRGB),
                                                        random(maxpenRGB),
                                                        random(maxpenRGB)));
           pens[i]:=createpen(ps_solid,random(maxpenwidth),temprgb); {create it}
         end;
     start_time:=getcurrenttime;      {get initial time, in msec}
end;


{timer_stuff is where the code goes that does something when the timer
 fires, typically drawing something on the screen, etc.  You will need
 to change this routine to do whatever you want to do...}

{Remember that, if you change this procedure's name or parameters, you will
 also need to modify the other routines which call it.  It is suggested that
 you leave the name, and parameters, alone, and provide for additional data
 stuff needed via global variables declared in the main program.}

procedure do_timer_stuff;   {executed when timer fires...}
var
   x1,y1:integer;            {coordinates for ellipse}
   x2,y2:integer;            {coordinates for ellipse}
   success:bool;             {flags successful drawing of ellipse}
   scr_time,elapsed:longint; {storage for time values}
   time_bytes:integer;       {number of bytes in time string}
   s:array [0..30] of char;  {string of elapsed time, in seconds}
begin
     selectobject(scrsvrhdc,blackfill);  {select pen to clear screen}
     moveto(scrsvrhdc,-1,-1);            {move pen beyond the origin}
     lineto(scrsvrhdc,xmax+1,ymax+1);    {draw line to clear screen...}
     SelectObject(scrsvrhdc,pens[random(maxpens)+1]); {select random pen...}
     success:=false;
     while not success do       {do while ellipse drawing is not successful...}
     begin
          x1:=random(xmax);     {get a new random coordinate for ellipse}
          y1:=random(ymax);     {get a new random coordinate for ellipse}
          x2:=random(xmax);     {get a new random coordinate for ellipse}
          y2:=random(ymax);     {get a new random coordinate for ellipse}
          success:=ellipse(scrsvrhdc,x1,y1,x2,y2); {try drawing the ellipse...}
     end;
     scr_time:=getcurrenttime;                {get current time, msec}
     elapsed:=(scr_time-start_time) div 1000; {get elapsed time, seconds}
     time_bytes:=wvsprintf(s,'%li',elapsed);  {format time}
     lstrcat(s,' seconds idle');              {add "seconds idle" to string}
     time_bytes:=time_bytes+13;               {update string byte count}
     TextOut(scrsvrhdc,(x1+x2) div 2,(y1+y2) div 2, {put the elapsed time...}
             s,time_bytes);                         {...(sec) on the screen}
end;


{Screen Saver finalization procedure.  Performs cleanup of whatever you
 have done within initialize_scr and do_timer_stuff, prior to program
 termination.  Any resources, etc., used by these procedures should be
 released before termination.}

{Remember that, if you change this procedure's name or parameters, you will
 also need to modify the other routines which call it.  It is suggested that
 you leave the name, and parameters, alone, and provide for additional data
 stuff needed via global variables declared in the main program.}

procedure finalize_scr;
var
   i:integer;           {loop index}
begin
     SelectObject(scrsvrhdc,0);    {deselect current pen}
     for i:=1 to maxpens do        {do for all pens created...}
         deleteobject(pens[i]);    {delete each pen}
     deleteobject(blackfill);      {also, delete the screen clearing pen}
     deleteobject(blackbrush);     {also, delete the screen clearing pen}
end;


{Windows expects this routine to be named "WindowProc", DO NOT change the
 name of this routine from "WindowProc" to anything else!!!  WindowProc
 handles all Windows messages the screen saver is interested in.}
{***** You should not need to modify ANY of the code in this block. *****}

function WindowProc(Window: HWnd; Message, WParam: Word;
                    LParam: Longint): Longint; export;
const
     firstmousemove:boolean=true; {flags mousemove as first, or otherwise}
begin
  WindowProc := 0;
  case Message of
    wm_keydown,wm_syskeydown,                     {handles the following: }
    wm_rbuttondown,wm_lbuttondown,wm_mbuttondown: {keydown/mousebutton clicks}
        postmessage(window,wm_close,0,0);         {post request to close}
    wm_mousemove:                               {handles mouse moves detected}
      if firstmousemove                         {first move of the mouse?}
         then firstmousemove:=false             {yes, just change flag}
         else postmessage(window,wm_close,0,0); {no, post request to close}
    wm_timer:                                   {handles timer message}
        do_timer_stuff;                         {calls screen routines}
    wm_Destroy:                                 {window is being destroyed}
        PostQuitMessage(0);                     {post request to terminate}
  end;
  WindowProc := DefWindowProc(Window, Message, WParam, LParam); {default proc}
end;


{WinMain - Windows expects this routine to be named "WinMain",
 DO NOT change the name of this routine from "WinMain"!!!}
{***** You should not need to modify ANY of the code in this block. *****}

procedure WinMain; {Windows expects this to be "WinMain" - DO NOT CHANGE!!}
var
   tempmousecount:integer;    {used to turn cursor/mousepointer on/off}
   savedmousecount:integer;   {cursor/mp initial state, to restore later}
   savedscrsvrstate:wordbool; {flags initial scrsvr state, also restored}
   timerID:integer;           {ID of screen saver timer created}
   Window:HWnd;               {handle of created window}
   Message:TMsg;              {windows messages}
const
  timeout:integer=500;        {timer timeout interval in milliseconds}
  svrname='Screen Saver';     {window class name, etc.}
  WindowClass: TWndClass = (
    style: 0;
    lpfnWndProc: @WindowProc;
    cbClsExtra: 0;
    cbWndExtra: 0;
    hInstance: 0;
    hIcon: 0;
    hCursor: 0;
    hbrBackground: 0;
    lpszMenuName: svrname;
    lpszClassName: svrname);

begin                        {Begin - WinMain}

{Set window class parameters, and register the class...}

  WindowClass.hInstance := HInstance;
  WindowClass.hbrBackground := GetStockObject(black_Brush);
  if not RegisterClass(WindowClass) then Halt(255);

{Create a new window...}

  Window := CreateWindow(
    svrname,
    nil,
    ws_OverlappedWindow,
    cw_UseDefault,
    cw_UseDefault,
    cw_UseDefault,
    cw_UseDefault,
    0,
    0,
    HInstance,
    nil);

{Remove the title/caption bar from the created window, get the current
 cursor/mousepointer visibility count, turn the mouse off, get the screen
 saver active state, and set the screen saver active state to off
 (to prevent Windows from trying to execute yet another screen saver).}

  SetWindowLong(window,GWL_STYLE, GetWindowLong(window,GWL_STYLE)
                                  AND NOT WS_CAPTION);  {Removes title bar}
  savedmousecount:=showcursor(false); {count - 1}
  savedmousecount:=showcursor(true);  {count - 1 + 1 = count (to be restored)}
  tempmousecount:=savedmousecount;    {copies the current count}
  while tempmousecount>=0 do tempmousecount:=showcursor(false);
  systemparametersinfo(16,0,@savedscrsvrstate,0); {state: TRUE = active}
  systemparametersinfo(17,0,nil,0);   {turn scr svr active state off}

{Show the window, maximized to size of screen, send update message to the
 window (for repainting, etc.), and make the window the topmost window.
 Also, store the window handle and handle to it's device context, call
 the screen saver initialization procedure, and set the timer.}

  ShowWindow(Window, sw_showmaximized); {show the window, maximized to screen}
  UpdateWindow(Window);                 {force any repainting...}
  SetWindowPos(window,hwnd_topmost,0,0,0,0,
               swp_noactivate+swp_nomove+swp_nosize);  {make it the topmost}
  scrsvrhwnd:=window;                   {store handle to window}
  scrsvrhdc:=getdc(scrsvrhwnd);         {get window device context handle}
  initialize_scr(timeout);              {do initializing specific to this svr}
  if (timeout>0) then timerID:=settimer(scrsvrhwnd,1,timeout,nil); {set timer}

{OK, now we are finished setting up, all that remains is to re-direct the
 windows messages, via a getmessage loop, until we are finished...}

  while GetMessage(Message, 0, 0, 0) do DispatchMessage(Message);

{Now, we are finished.  Execute the finalize_scr procedure to do the
 cleanup after code specific to this screen saver only.  Restore the
 cursor/mousepointer to it's original state, restore the screen saver
 active state to it's initial state, release the device context stuff,
 kill the timer, destroy window, and halt/terminate the application.}

  finalize_scr;
  while tempmousecount<savedmousecount do tempmousecount:=showcursor(true);
  if savedscrsvrstate then systemparametersinfo(17,1,nil,0);
  ReleaseDC(scrsvrhwnd,scrsvrhdc);
  if (timeout>0) then KillTimer(scrsvrhwnd,timerID);
  DestroyWindow(scrsvrhwnd);
  Halt;

end;                         {End   - WinMain}


{Main screen saver program body.}
{***** You should not need to modify ANY of the code in this block. *****}

begin
  if HPrevInst<>0 then halt;      {previous instance is running...}
  c:=paramstr(1);                 {get the command string parameter}
  if (c<>'/s') and (c<>'/S') and  {these conditions indicate that...}
     (c<>'-s') and (c<>'-S') then {...scrnsavr configure mode is needed}
     begin                        {default is scrsvr configure mode...}
          configure_scr;          {call procedure to configure scrsvr}
          halt;                   {scrnsavr is now configured... Halt}
     end;                         {end scrnsavr configure block...}
  WinMain;                        {call WinMain for scrnsavr execution}
end.
